
// This file has been generated by the GUI designer. Do not modify.
namespace WideMargin.GUI
{
	public partial class AboutDialog
	{
		private global::WideMargin.GUI.RichTextView _richTextView;
		private global::Gtk.Button buttonCancel;
		
		protected virtual void Build ()
		{
			global::Stetic.Gui.Initialize (this);
			// Widget WideMargin.GUI.AboutDialog
			this.Name = "WideMargin.GUI.AboutDialog";
			this.Title = global::Mono.Unix.Catalog.GetString ("About Wide Margin");
			this.WindowPosition = ((global::Gtk.WindowPosition)(4));
			this.Modal = true;
			// Internal child WideMargin.GUI.AboutDialog.VBox
			global::Gtk.VBox w1 = this.VBox;
			w1.Name = "dialog1_VBox";
			w1.BorderWidth = ((uint)(2));
			// Container child dialog1_VBox.Gtk.Box+BoxChild
			this._richTextView = new global::WideMargin.GUI.RichTextView ();
			this._richTextView.Events = ((global::Gdk.EventMask)(256));
			this._richTextView.Name = "_richTextView";
			w1.Add (this._richTextView);
			global::Gtk.Box.BoxChild w2 = ((global::Gtk.Box.BoxChild)(w1 [this._richTextView]));
			w2.Position = 0;
			// Internal child WideMargin.GUI.AboutDialog.ActionArea
			global::Gtk.HButtonBox w3 = this.ActionArea;
			w3.Name = "dialog1_ActionArea";
			w3.Spacing = 10;
			w3.BorderWidth = ((uint)(5));
			w3.LayoutStyle = ((global::Gtk.ButtonBoxStyle)(4));
			// Container child dialog1_ActionArea.Gtk.ButtonBox+ButtonBoxChild
			this.buttonCancel = new global::Gtk.Button ();
			this.buttonCancel.CanDefault = true;
			this.buttonCancel.CanFocus = true;
			this.buttonCancel.Name = "buttonCancel";
			this.buttonCancel.Label = global::Mono.Unix.Catalog.GetString ("Close");
			this.AddActionWidget (this.buttonCancel, -7);
			global::Gtk.ButtonBox.ButtonBoxChild w4 = ((global::Gtk.ButtonBox.ButtonBoxChild)(w3 [this.buttonCancel]));
			w4.Expand = false;
			w4.Fill = false;
			if ((this.Child != null)) {
				this.Child.ShowAll ();
			}
			this.DefaultWidth = 428;
			this.DefaultHeight = 371;
			this.Show ();
		}
	}
}
