
// This file has been generated by the GUI designer. Do not modify.
namespace WideMargin.GUI
{
	public partial class SearchPage
	{
		private global::Gtk.VBox vbox4;
		private global::Gtk.HBox hbox1;
		private global::Gtk.Button _backButton;
		private global::Gtk.Button _forwardButton;
		private global::Gtk.Entry _bibleBar;
		private global::Gtk.Button _menuButton;
		private global::Gtk.HButtonBox hbuttonbox2;
		
		protected virtual void Build ()
		{
			global::Stetic.Gui.Initialize (this);
			// Widget WideMargin.GUI.SearchPage
			global::Stetic.BinContainer.Attach (this);
			this.Name = "WideMargin.GUI.SearchPage";
			// Container child WideMargin.GUI.SearchPage.Gtk.Container+ContainerChild
			this.vbox4 = new global::Gtk.VBox ();
			this.vbox4.Name = "vbox4";
			// Container child vbox4.Gtk.Box+BoxChild
			this.hbox1 = new global::Gtk.HBox ();
			this.hbox1.Name = "hbox1";
			this.hbox1.Spacing = 2;
			// Container child hbox1.Gtk.Box+BoxChild
			this._backButton = new global::Gtk.Button ();
			this._backButton.CanFocus = true;
			this._backButton.Name = "_backButton";
			this._backButton.UseUnderline = true;
			this._backButton.Relief = ((global::Gtk.ReliefStyle)(2));
			// Container child _backButton.Gtk.Container+ContainerChild
			global::Gtk.Alignment w1 = new global::Gtk.Alignment (0.5F, 0.5F, 0F, 0F);
			// Container child GtkAlignment.Gtk.Container+ContainerChild
			global::Gtk.HBox w2 = new global::Gtk.HBox ();
			w2.Spacing = 2;
			// Container child GtkHBox.Gtk.Container+ContainerChild
			global::Gtk.Image w3 = new global::Gtk.Image ();
			w2.Add (w3);
			// Container child GtkHBox.Gtk.Container+ContainerChild
			global::Gtk.Label w5 = new global::Gtk.Label ();
			w2.Add (w5);
			w1.Add (w2);
			this._backButton.Add (w1);
			this.hbox1.Add (this._backButton);
			global::Gtk.Box.BoxChild w9 = ((global::Gtk.Box.BoxChild)(this.hbox1 [this._backButton]));
			w9.Position = 0;
			w9.Expand = false;
			w9.Fill = false;
			// Container child hbox1.Gtk.Box+BoxChild
			this._forwardButton = new global::Gtk.Button ();
			this._forwardButton.CanFocus = true;
			this._forwardButton.Name = "_forwardButton";
			this._forwardButton.UseUnderline = true;
			this._forwardButton.Relief = ((global::Gtk.ReliefStyle)(2));
			// Container child _forwardButton.Gtk.Container+ContainerChild
			global::Gtk.Alignment w10 = new global::Gtk.Alignment (0.5F, 0.5F, 0F, 0F);
			// Container child GtkAlignment.Gtk.Container+ContainerChild
			global::Gtk.HBox w11 = new global::Gtk.HBox ();
			w11.Spacing = 2;
			// Container child GtkHBox.Gtk.Container+ContainerChild
			global::Gtk.Image w12 = new global::Gtk.Image ();
			w11.Add (w12);
			// Container child GtkHBox.Gtk.Container+ContainerChild
			global::Gtk.Label w14 = new global::Gtk.Label ();
			w11.Add (w14);
			w10.Add (w11);
			this._forwardButton.Add (w10);
			this.hbox1.Add (this._forwardButton);
			global::Gtk.Box.BoxChild w18 = ((global::Gtk.Box.BoxChild)(this.hbox1 [this._forwardButton]));
			w18.Position = 1;
			w18.Expand = false;
			w18.Fill = false;
			// Container child hbox1.Gtk.Box+BoxChild
			this._bibleBar = new global::Gtk.Entry ();
			this._bibleBar.CanFocus = true;
			this._bibleBar.Name = "_bibleBar";
			this._bibleBar.IsEditable = true;
			this._bibleBar.InvisibleChar = 'n';
			this.hbox1.Add (this._bibleBar);
			global::Gtk.Box.BoxChild w19 = ((global::Gtk.Box.BoxChild)(this.hbox1 [this._bibleBar]));
			w19.Position = 2;
			// Container child hbox1.Gtk.Box+BoxChild
			this._menuButton = new global::Gtk.Button ();
			this._menuButton.CanFocus = true;
			this._menuButton.Name = "_menuButton";
			this._menuButton.UseUnderline = true;
			this._menuButton.Relief = ((global::Gtk.ReliefStyle)(2));
			// Container child _menuButton.Gtk.Container+ContainerChild
			global::Gtk.Alignment w20 = new global::Gtk.Alignment (0.5F, 0.5F, 0F, 0F);
			// Container child GtkAlignment.Gtk.Container+ContainerChild
			global::Gtk.HBox w21 = new global::Gtk.HBox ();
			w21.Spacing = 2;
			// Container child GtkHBox.Gtk.Container+ContainerChild
			global::Gtk.Image w22 = new global::Gtk.Image ();
			w21.Add (w22);
			// Container child GtkHBox.Gtk.Container+ContainerChild
			global::Gtk.Label w24 = new global::Gtk.Label ();
			w21.Add (w24);
			w20.Add (w21);
			this._menuButton.Add (w20);
			this.hbox1.Add (this._menuButton);
			global::Gtk.Box.BoxChild w28 = ((global::Gtk.Box.BoxChild)(this.hbox1 [this._menuButton]));
			w28.Position = 3;
			w28.Expand = false;
			w28.Fill = false;
			this.vbox4.Add (this.hbox1);
			global::Gtk.Box.BoxChild w29 = ((global::Gtk.Box.BoxChild)(this.vbox4 [this.hbox1]));
			w29.Position = 0;
			w29.Expand = false;
			w29.Fill = false;
			// Container child vbox4.Gtk.Box+BoxChild
			this.hbuttonbox2 = new global::Gtk.HButtonBox ();
			this.hbuttonbox2.Name = "hbuttonbox2";
			this.vbox4.Add (this.hbuttonbox2);
			global::Gtk.Box.BoxChild w30 = ((global::Gtk.Box.BoxChild)(this.vbox4 [this.hbuttonbox2]));
			w30.Position = 1;
			w30.Expand = false;
			w30.Fill = false;
			this.Add (this.vbox4);
			if ((this.Child != null)) {
				this.Child.ShowAll ();
			}
			this.Hide ();
			this._backButton.Clicked += new global::System.EventHandler (this.HandleBackButtonClicked);
			this._forwardButton.Clicked += new global::System.EventHandler (this.HandleForwardButtonClicked);
			this._menuButton.Clicked += new global::System.EventHandler (this.HandleMenuClicked);
		}
	}
}
