using System;
using NUnit.Framework;
using System.Linq;
using System.Xml.Linq;
using System.Collections.Generic;
using System.Collections;


namespace WideMargin.Database.Tests
{	
	[TestFixture()]
	public class Test
	{
		[Test()]
		public void TestCase ()
		{

		}
		

	}
}

