"c:\Program Files (x86)\Windows Installer XML v3.5\bin\candle.exe" widemargin.wxs -out obj\widemargin.wixobj
"c:\Program Files (x86)\Windows Installer XML v3.5\bin\light.exe" obj\widemargin.wixobj -out bin\widemargin.msi -ext WixUIExtension -ext WixNetFxExtension

"c:\Program Files (x86)\dotNetInstaller\bin\InstallerLinker.exe" /Configuration:bootstrapperConfig.xml /Output:bin\WideMarginInstaller.exe /Template:"C:\Program Files (x86)\dotNetInstaller\bin\dotNetInstaller.exe" /Icon:..\icon\WindowsIcon\widemargin.ico