// WideMargin. Simple fast bible software.
// Copyright (C) 2011  Daniel Hughes
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;

namespace WideMargin.MVCInterfaces
{
	public interface IPassageView
	{
		event EventHandler<EventArgs> RequestNext;
		
		event EventHandler<EventArgs> RequestPrevious;
		
		void ShowVerses(IEnumerable<IVerse> searchResults);
		
		event EventHandler<LinkClickedEventArgs<IVerse>> VerseLinkActivated;
		
		/// <summary>
		/// Scrolls to the specified verse
		/// </summary>
		/// <param name="verse">Verse to scroll to</param>
		void ScrollToVerse(IVerse verse);
	}
}

