
// This file has been generated by the GUI designer. Do not modify.
namespace WideMargin.GUI
{
	public partial class FilteredView
	{
		protected virtual void Build ()
		{
			global::Stetic.Gui.Initialize (this);
			// Widget WideMargin.GUI.FilteredView
			global::Stetic.BinContainer.Attach (this);
			this.Name = "WideMargin.GUI.FilteredView";
			if ((this.Child != null)) {
				this.Child.ShowAll ();
			}
			this.Hide ();
		}
	}
}
