
// This file has been generated by the GUI designer. Do not modify.
namespace WideMargin.GUI
{
	public partial class PassagePage
	{
		private global::Gtk.VBox _vbox;
		private global::Gtk.HBox _buttonBox;
		private global::Gtk.Fixed fixed3;
		private global::Gtk.Button _previousButton;
		private global::Gtk.Button _nextButton;
		private global::Gtk.Fixed fixed2;
		
		protected virtual void Build ()
		{
			global::Stetic.Gui.Initialize (this);
			// Widget WideMargin.GUI.PassagePage
			global::Stetic.BinContainer.Attach (this);
			this.Name = "WideMargin.GUI.PassagePage";
			// Container child WideMargin.GUI.PassagePage.Gtk.Container+ContainerChild
			this._vbox = new global::Gtk.VBox ();
			this._vbox.Name = "_vbox";
			this._vbox.Spacing = 1;
			// Container child _vbox.Gtk.Box+BoxChild
			this._buttonBox = new global::Gtk.HBox ();
			this._buttonBox.Name = "_buttonBox";
			this._buttonBox.Spacing = 6;
			// Container child _buttonBox.Gtk.Box+BoxChild
			this.fixed3 = new global::Gtk.Fixed ();
			this.fixed3.Name = "fixed3";
			this.fixed3.HasWindow = false;
			this._buttonBox.Add (this.fixed3);
			global::Gtk.Box.BoxChild w1 = ((global::Gtk.Box.BoxChild)(this._buttonBox [this.fixed3]));
			w1.Position = 0;
			// Container child _buttonBox.Gtk.Box+BoxChild
			this._previousButton = new global::Gtk.Button ();
			this._previousButton.CanFocus = true;
			this._previousButton.Name = "_previousButton";
			this._previousButton.UseUnderline = true;
			this._previousButton.Relief = ((global::Gtk.ReliefStyle)(2));
			// Container child _previousButton.Gtk.Container+ContainerChild
			global::Gtk.Alignment w2 = new global::Gtk.Alignment (0.5F, 0.5F, 0F, 0F);
			// Container child GtkAlignment.Gtk.Container+ContainerChild
			global::Gtk.HBox w3 = new global::Gtk.HBox ();
			w3.Spacing = 2;
			// Container child GtkHBox.Gtk.Container+ContainerChild
			global::Gtk.Image w4 = new global::Gtk.Image ();
			w3.Add (w4);
			// Container child GtkHBox.Gtk.Container+ContainerChild
			global::Gtk.Label w6 = new global::Gtk.Label ();
			w3.Add (w6);
			w2.Add (w3);
			this._previousButton.Add (w2);
			this._buttonBox.Add (this._previousButton);
			global::Gtk.Box.BoxChild w10 = ((global::Gtk.Box.BoxChild)(this._buttonBox [this._previousButton]));
			w10.Position = 1;
			w10.Expand = false;
			w10.Fill = false;
			// Container child _buttonBox.Gtk.Box+BoxChild
			this._nextButton = new global::Gtk.Button ();
			this._nextButton.CanFocus = true;
			this._nextButton.Name = "_nextButton";
			this._nextButton.UseUnderline = true;
			this._nextButton.Relief = ((global::Gtk.ReliefStyle)(2));
			// Container child _nextButton.Gtk.Container+ContainerChild
			global::Gtk.Alignment w11 = new global::Gtk.Alignment (0.5F, 0.5F, 0F, 0F);
			// Container child GtkAlignment.Gtk.Container+ContainerChild
			global::Gtk.HBox w12 = new global::Gtk.HBox ();
			w12.Spacing = 2;
			// Container child GtkHBox.Gtk.Container+ContainerChild
			global::Gtk.Image w13 = new global::Gtk.Image ();
			w12.Add (w13);
			// Container child GtkHBox.Gtk.Container+ContainerChild
			global::Gtk.Label w15 = new global::Gtk.Label ();
			w12.Add (w15);
			w11.Add (w12);
			this._nextButton.Add (w11);
			this._buttonBox.Add (this._nextButton);
			global::Gtk.Box.BoxChild w19 = ((global::Gtk.Box.BoxChild)(this._buttonBox [this._nextButton]));
			w19.Position = 2;
			w19.Expand = false;
			w19.Fill = false;
			// Container child _buttonBox.Gtk.Box+BoxChild
			this.fixed2 = new global::Gtk.Fixed ();
			this.fixed2.Name = "fixed2";
			this.fixed2.HasWindow = false;
			this._buttonBox.Add (this.fixed2);
			global::Gtk.Box.BoxChild w20 = ((global::Gtk.Box.BoxChild)(this._buttonBox [this.fixed2]));
			w20.Position = 3;
			this._vbox.Add (this._buttonBox);
			global::Gtk.Box.BoxChild w21 = ((global::Gtk.Box.BoxChild)(this._vbox [this._buttonBox]));
			w21.Position = 1;
			w21.Expand = false;
			w21.Fill = false;
			this.Add (this._vbox);
			if ((this.Child != null)) {
				this.Child.ShowAll ();
			}
			this.Hide ();
			this._previousButton.Clicked += new global::System.EventHandler (this.PreviousButton_Clicked);
			this._nextButton.Clicked += new global::System.EventHandler (this.NextButton_Clicked);
		}
	}
}
